require 'spec_helper'
require 'hashie'
require 'faraday'

describe Facile::Api::HttpMethods do
  before do
    class TestClass
      include ::Facile::Api
      url 'http://bbc.co.uk'
    end
  end

  context 'able to set values' do
    it 'http methods work' do
      # there's no real need to test this any further
      expect(TestClass.url).to eq('http://bbc.co.uk')
    end
  end

  context 'does not share state' do
    it 'other classes using the same method have different values' do
      class Zomg
        include ::Facile::Api
      end

      expect(TestClass.url).to eq('http://bbc.co.uk')
      expect(Zomg.url).to eq(nil)
    end
  end
end

describe Facile::Api::RequestMethods do
  before do
    class TestClass
      include ::Facile::Api

      url 'http://bbc.co.uk'
    end
  end

  context 'request options' do
    it 'can be generated' do
      expected = {
        :url => 'http://bbc.co.uk',
        :method => :get,
        :headers => {},
        :params => {},
        :body => nil,
        :options => {}
      }

      expect(TestClass.request_options(TestClass.url)).to eq(expected)
    end
  end

  context 'generate url' do
    it 'can insert placeholder' do
      class OtherTestClass
        include ::Facile::Api

        url 'http://bbc.co.uk/{id}'
      end

      generated_url = OtherTestClass.generate_url(1, OtherTestClass.url)

      expect(generated_url).to eq('http://bbc.co.uk/1')
    end
  end

  context 'request things' do
    it 'can request things' do
      faraday_response = Faraday::Response.new({
        :method => :get,
        :response_headers => {'content-type' => 'text/plain'},
        :body => 'foo',
        :url => 'http://bbc.com'
      })

      Faraday::Connection.any_instance.stub(:get => faraday_response)

      inst = TestClass.find()

      expect(inst).to be_kind_of(TestClass)
      expect(inst.response).to eq('foo')
    end
  end
end

describe Facile::Api::ForwardMethods do
  before do
    class TestClass
      include ::Facile::Api
    end
  end

  context 'can create instance' do
    it 'create instance of test class' do
      inst = TestClass.create_instance

      expect(inst).to be_kind_of(TestClass)
    end

    it 'can create instance with string data' do
      inst = TestClass.create_instance('test')

      expect(inst.response).to eq('test')
    end

    it 'can create instance with hash' do
      hash = Hashie::Mash.new({:foo => 'bar'})
      inst = TestClass.create_instance(hash)

      expect(inst.response).to eq(hash)
    end
  end

  context 'can delegate methods' do
    it 'can read from a method' do
      hash = Hashie::Mash.new({:foo => 'bar'})
      inst = TestClass.create_instance(hash)

      expect(inst.foo).to eq('bar')
    end

    it 'can write from a method' do
      hash = Hashie::Mash.new({:foo => 'bar'})
      inst = TestClass.create_instance(hash)

      expect(inst.foo).to eq('bar')

      inst.foo = 'zomg'
      expect(inst.foo).to eq('zomg')
    end
  end
end
